/**
 * Copyright 2005-2015 Alfresco Software, Ltd. All rights reserved.
 * License rights for this program may be obtained from Alfresco Software, Ltd.
 * pursuant to a written agreement and any use of this program without such an
 * agreement is prohibited.
 */
package com.activiti;


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;

/**
 * @author Joram Barrez
 */
public class BackupEvents {

	private static String jdbcUrl;
	private static String jdbcUsername;
	private static String jdbcPassword;
	private static String tablePrefix;
	private static String outputFile;
	private static Long batchSize;

	public static void main(String[] args) throws Exception {

		System.out.println("Usage:");
		System.out.println("-DjdbcUrl=<jdbcUrl> -DjdbcUsername=<jdbcUsername> -DjdbcPassword<jdbcPassword> -DtablePrefix=<tablePrefix> -DbatchSize=<batchSize> -DoutputFile=<outputFile>");
		System.out.println();

		jdbcUrl = System.getProperty("jdbcUrl");
		jdbcUsername = System.getProperty("jdbcUsername");
		jdbcPassword = System.getProperty("jdbcPassword");
		tablePrefix = System.getProperty("tablePrefix");
		outputFile = System.getProperty("outputFile") != null ? System.getProperty("outputFile") : "output.csv";
		batchSize = System.getProperty("batchSize") != null ? Long.valueOf(System.getProperty("batchSize")) : 250;
		
		if (jdbcUrl == null) {
			throw new RuntimeException("Jdbc url is mandatory");
		}
		if (jdbcUsername == null) {
			throw new RuntimeException("Jdbc url is mandatory");
		}
		if (jdbcPassword == null) {
			throw new RuntimeException("Jdbc url is mandatory");
		}

		printIfNotNull("Jdbc url", jdbcUrl);
		printIfNotNull("Jdbc username", jdbcUsername);
		printIfNotNull("Jdbc password", jdbcPassword);
		printIfNotNull("Table prefix", tablePrefix);
		printIfNotNull("output file", outputFile);
		printIfNotNull("Batch size", batchSize);

		long minLogNr = getMinLogNr();
		long maxLogNr = getMaxLogNr();
		System.out.println("Start log number = " + minLogNr + ", last log nr = " + maxLogNr);
		
		File file = new File(outputFile);
		if (!file.createNewFile()) {
			throw new RuntimeException("File " + outputFile + " already exists");
		}
		
		FileWriter fileWriter = new FileWriter(outputFile);
		BufferedWriter bufferedWriter = new BufferedWriter(fileWriter);
		CSVPrinter csvFilePrinter = new CSVPrinter(bufferedWriter, CSVFormat.DEFAULT.withRecordSeparator(System.lineSeparator()));

		csvFilePrinter.printRecord(Constants.HEADER);

		for (long currentLogNr = minLogNr; currentLogNr <= maxLogNr; currentLogNr += batchSize) {
			
			System.out.println("Writing log events for logNr [" + currentLogNr + "-" + (currentLogNr+batchSize) + "]");

			ResultSet resultSet = getBatch(currentLogNr, currentLogNr + batchSize);
			while (resultSet.next()) {
				Long logNr = resultSet.getLong("LOG_NR_");
				String type = resultSet.getString("TYPE_");
				String processDefinitionId = resultSet.getString("PROC_DEF_ID_");
				String processInstanceId = resultSet.getString("PROC_INST_ID_");
				String executionId = resultSet.getString("EXECUTION_ID_");
				String taskId = resultSet.getString("TASK_ID_");
				Date timeStamp = resultSet.getDate("TIME_STAMP_");
				String userId = resultSet.getString("USER_ID_");
				String data = convert(resultSet.getClob("DATA_"));
	
				List<Object> csvRecord = new ArrayList<Object>();
				csvRecord.add(logNr);
				csvRecord.add(type);
				csvRecord.add(processDefinitionId);
				csvRecord.add(processInstanceId);
				csvRecord.add(executionId);
				csvRecord.add(taskId);
				csvRecord.add(Constants.DATE_FORMAT.format(timeStamp));
				csvRecord.add(userId);
				csvRecord.add(data);
				csvFilePrinter.printRecord(csvRecord);
			}

		}
		
		bufferedWriter.flush();
		bufferedWriter.close();
		csvFilePrinter.close();

		System.out.println("All Done");
	}

	private static void printIfNotNull(String helpText, Object o) {
		if (o != null) {
			System.out.println(helpText + " : " + o);
		}
	}

	private static long getMinLogNr() throws SQLException {
		Connection connection = DriverManager.getConnection(jdbcUrl, jdbcUsername, jdbcPassword);
		PreparedStatement preparedStatement = connection.prepareStatement("select MIN(LOG_NR_) from "
		        + (tablePrefix != null ? ("." + tablePrefix) : "") + "PROCESSED_ACTIVITI_EVENTS");
		ResultSet minimalLogNr = preparedStatement.executeQuery();
		minimalLogNr.next();
		return minimalLogNr.getLong(1);
	}

	private static long getMaxLogNr() throws SQLException {
		Connection connection = DriverManager.getConnection(jdbcUrl, jdbcUsername, jdbcPassword);
		PreparedStatement preparedStatement = connection.prepareStatement("select MAX(LOG_NR_) from "
		        + (tablePrefix != null ? ("." + tablePrefix) : "") + "PROCESSED_ACTIVITI_EVENTS");
		ResultSet maxLogNr = preparedStatement.executeQuery();
		maxLogNr.next();
		return maxLogNr.getLong(1);
	}

	private static ResultSet getBatch(long start, long end) throws SQLException {
		Connection connection = DriverManager.getConnection(jdbcUrl, jdbcUsername, jdbcPassword);
		PreparedStatement preparedStatement = connection.prepareStatement("select * from "
		        + (tablePrefix != null ? ("." + tablePrefix) : "") + "PROCESSED_ACTIVITI_EVENTS where LOG_NR_ >= " + start
		        + " and LOG_NR_ < " + end);
		return preparedStatement.executeQuery();
	}

	private static String convert(Clob clob) throws SQLException, IOException {
		Reader reader = new BufferedReader(clob.getCharacterStream());
		StringBuilder strb = new StringBuilder();
		char[] buffer = new char[8192];
		int nrRead = reader.read(buffer);
		while (nrRead > 0) {
			strb.append(new String(buffer, 0, nrRead));
			nrRead = reader.read(buffer);
		}
		return strb.toString();
	}

}
